#!/usr/bin/R Rscript

Args <- commandArgs(TRUE)
#print("Usage: x.exe   input_path   window_len  filter  output_filename\n")
##setting input path,the first parameter
data_orgin <- read.table(Args[1],header = FALSE)

classI_A <- which(data_orgin[,1]=="A")
classI_B <- which(data_orgin[,1]=="B")
classI_C <- which(data_orgin[,1]=="C")
classI <- c(classI_A,classI_B,classI_C)
data_orgin_I <- data_orgin[classI,]
data_orgin_II <- data_orgin[-classI,]

##processing class I data, divide into groups
alleles_group_I <- unique(data_orgin_I[,2])
out_alleles_I <- matrix(NA,length(unique(data_orgin[,3])),length(alleles_group_I))
colnames(out_alleles_I) <- as.character(alleles_group_I)
##setting the size of window, the second parameter
winlen <- as.numeric(Args[2])

##setting filter criteria, the third parameter
filter <- as.numeric(Args[3])

##setting output file name, the fourth parameter
out_filename <- Args[4]

##Save the matrix
tmp_matrix_I <- numeric(0)

g <- 1
for(g in 1:length(alleles_group_I)){
  lines1 <- which(data_orgin_I[,2]==alleles_group_I[g])
  data <- data_orgin_I[lines1,3:5]
  data1 <- as.matrix(data[,1])
  data2 <- as.matrix(data[,2:3])
  
  lines2 <- which(data2[,2]==-1)
  data2[lines2,2] <- NA
  
  alleles_unique <- unique(data1[,1])
  num_row <- nrow(data)/length(alleles_unique)
  new.matrix <- matrix(NA,num_row,length(alleles_unique))
  colnames(new.matrix) <- alleles_unique
  
  i <- 1
  for(i in 1:length(alleles_unique)){
    nr1 <- which(data1[,1]==alleles_unique[i])
    nc1 <- which(colnames(new.matrix)==alleles_unique[i])
    new.matrix[,nc1] <- data2[nr1,2]
  }
  
  ##Generates a new matrix based on the window size
  nr2 <- nrow(new.matrix)-winlen+1
  nc3 <- ncol(new.matrix)
  new.matrix1 <- matrix(NA,nr2,nc3)
  colnames(new.matrix1) <- colnames(new.matrix)
  q <- 1
  for(q in 1:nc3){
    u <- 1
    for(u in 1:nr2){
      new.matrix1[u,q] <- sum(new.matrix[u:(u+winlen-1),q])
    }
  }
  
  ##Replace the missing value with -1
  loc <- 1
  for(loc in 1:ncol(new.matrix1)){
    r <- 1
    for(r in 1:nrow(new.matrix1)){
      if(is.na(new.matrix1[r,loc])){
        new.matrix1[r,loc] <- -1
      }
    }
  }
  
  ##Combine the newly generated matrix into one
  tmp_matrix_I <- cbind(tmp_matrix_I,new.matrix1)
  
  ##Replace the -1 with average value
  p <- 1
  for(p in 1:nrow(new.matrix1)){
    column <- which(new.matrix1[p,]==-1)
    if(length(column) < ncol(new.matrix1)){
      ave <- mean(new.matrix1[p,-column])
      new.matrix1[p,column] <- ave
    }
  }
  
  ##Counts each column less than the average and Equal to 0,then take the maximum
  max_matrix <- matrix(NA,1,ncol(new.matrix1))
  colnames(max_matrix) <- colnames(new.matrix1)
  line <- 1
  for(line in 1:ncol(new.matrix1)){
    tmp_num <- 0
    max_num <- 0
    r <- 1
    for(r in 1:nrow(new.matrix1)){
      ave <- mean(new.matrix1[r,])
      if(new.matrix1[r,line]< ave && new.matrix1[r,line]==0){
        tmp_num <- tmp_num+1
      }
      else{
        if(tmp_num>max_num){
          max_num <- tmp_num
        }
        tmp_num <- 0
      }
    }
    max_matrix[1,line] <- max_num
  }
  max_matrix <- t(as.matrix(max_matrix[,order(max_matrix[1,],decreasing = F)]))
  
  ##Extract the first three alleles names
  name_matrix1 <- matrix(NA,1,ncol(max_matrix))
  uq_num <- length(unique(max_matrix[1,]))
  if(uq_num <=3){
    name_matrix1 <- colnames(max_matrix)
  }else{
    uq_len <- length(which(max_matrix[1,]==unique(max_matrix[1,])[1] | max_matrix[1,]==unique(max_matrix[1,])[2] | max_matrix[1,]==unique(max_matrix[1,])[3]))
    name_matrix1[1,1:uq_len] <- colnames(max_matrix)[which(max_matrix[1,]==unique(max_matrix[1,])[1] | max_matrix[1,]==unique(max_matrix[1,])[2] | max_matrix[1,]==unique(max_matrix[1,])[3])] 
  }
  name_matrix11 <- matrix(NA,1,length(name_matrix1))
  for(nm1 in 1:length(name_matrix1)){
    name_matrix11[1,nm1] <- name_matrix1[nm1]
  }
  name_matrix1 <- name_matrix11
  ##Extraction of alleles that meet the filtration criteria
  num.col <- c()
  nc4 <- 1
  for(nc4 in 1:ncol(new.matrix1)){
    nr3 <- 1
    for(nr3 in 1:(nrow(new.matrix1)-filter)){
      t <- 0
      linenum <- 1
      new.matrix2 <- matrix(0,filter+1,1)
      for(t in nr3:(nr3+filter)){
        if(new.matrix1[t,nc4]< mean(new.matrix1[t,])){
          if(new.matrix1[t,nc4]==0){
            new.matrix2[linenum,1] <- 1
          }
        }
        linenum <- linenum+1
      }
        
      if(sum(new.matrix2[,1])==filter+1){
        num.col <- c(num.col,nc4)
         break
      }
    }
  }
  if(length(num.col)==0){
    new.matrix3 <- new.matrix1
  }else if(ncol(new.matrix1)-length(num.col)==1){
    new.matrix3 <- as.matrix(new.matrix1[,-num.col])
    colnames(new.matrix3) <- colnames(new.matrix1)[-num.col]
  }else{
    new.matrix3 <- new.matrix1[,-num.col]
  }
  
  if(ncol(new.matrix3) > 0){ 
    name_matrix2 <- matrix(NA,1,ncol(new.matrix3))
    name_matrix2 <- colnames(new.matrix3)
  }else{
    name_matrix2 <- c()
  }
  ##Merge alleles which are extracted through two standards
  if(length(name_matrix2)>0){
    name_matrix2 <- t(as.matrix(name_matrix2))
    name_matrix <- union(name_matrix1[1,which(!is.na(name_matrix1[1,]))],name_matrix2[1,which(!is.na(name_matrix2[1,]))])
  }else{
    name_matrix <- name_matrix1[1,which(!is.na(name_matrix1[1,]))]
  }
  name_matrix <- t(as.matrix(name_matrix))
  
  ##The names are merged if the top three of name is identical 
  uq_len1 <- ncol(name_matrix)
  new_name_matrix <- matrix(NA,uq_len1,1)
  tmp <- 1
  for(tmp in 1:uq_len1){
    len <- length(strsplit(name_matrix[tmp],':',fixed = TRUE)[[1]])
    if(len > 3){
      new_name_matrix[tmp,1] <- paste(strsplit(name_matrix[tmp],':',fixed = TRUE)[[1]][1:3],collapse = ":")
    }else{
      new_name_matrix[tmp,1] <- name_matrix[tmp]
    }
  }
  T <- 1
  for(T in 1:length(unique(new_name_matrix[,1]))){
    Tlines <-  which(new_name_matrix[,1]==unique(new_name_matrix[,1])[T])
    out_alleles_I[T,which(colnames(out_alleles_I)==alleles_group_I[g])] <- name_matrix[1,Tlines[1]]
  }
}

###processing class II data, do not divide into groups
alleles_group_II <- unique(data_orgin_II[,1])
out_alleles_II <- matrix(NA,length(unique(data_orgin[,3])),length(alleles_group_II))
colnames(out_alleles_II) <- as.character(alleles_group_II)

tmp_matrix_II <- numeric(0)
g <- 1
for(g in 1:length(alleles_group_II)){
  lines1 <- which(data_orgin_II[,1]==alleles_group_II[g])
  data <- data_orgin_II[lines1,3:5]
  data1 <- as.matrix(data[,1])
  data2 <- as.matrix(data[,2:3])
  
  lines2 <- which(data2[,2]==-1)
  data2[lines2,2] <- NA
  
  alleles_unique <- unique(data1[,1])
  num_row <- nrow(data)/length(alleles_unique)
  new.matrix <- matrix(NA,num_row,length(alleles_unique))
  colnames(new.matrix) <- alleles_unique
  
  i <- 1
  for(i in 1:length(alleles_unique)){
    nr1 <- which(data1[,1]==alleles_unique[i])
    nc1 <- which(colnames(new.matrix)==alleles_unique[i])
    new.matrix[,nc1] <- data2[nr1,2]
  }
  
  nr2 <- nrow(new.matrix)-winlen+1
  nc3 <- ncol(new.matrix)
  new.matrix1 <- matrix(NA,nr2,nc3)
  colnames(new.matrix1) <- colnames(new.matrix)
  
  q <- 1
  for(q in 1:nc3){
    u <- 1
    for(u in 1:nr2){
      new.matrix1[u,q] <- sum(new.matrix[u:(u+winlen-1),q])
    }
  }
  
  loc <- 1
  for(loc in 1:ncol(new.matrix1)){
    r <- 1
    for(r in 1:nrow(new.matrix1)){
      if(is.na(new.matrix1[r,loc])){
        new.matrix1[r,loc] <- -1
      }
    }
  }
  tmp_matrix_II <- cbind(tmp_matrix_II,new.matrix1)
  
  p <- 1
  for(p in 1:nrow(new.matrix1)){
    column <- which(new.matrix1[p,]==-1)
    if(length(column) < ncol(new.matrix1)){
      ave <- mean(new.matrix1[p,-column])
      new.matrix1[p,column] <- ave
    }
  }
  
  max_matrix <- matrix(NA,1,ncol(new.matrix1))
  colnames(max_matrix) <- colnames(new.matrix1)
  line <- 1
  for(line in 1:ncol(new.matrix1)){
    tmp_num <- 0
    max_num <- 0
    r <- 1
    for(r in 1:nrow(new.matrix1)){
      ave <- mean(new.matrix1[r,])
      if(new.matrix1[r,line]< ave && new.matrix1[r,line]==0){
        tmp_num <- tmp_num+1
      }
      else{
        if(tmp_num>max_num){
          max_num <- tmp_num
        }
        tmp_num <- 0
      }
    }
    max_matrix[1,line] <- max_num
  }
  
  max_matrix <- t(as.matrix(max_matrix[,order(max_matrix[1,],decreasing = F)]))
  
  ##Extract the first three alleles names
  name_matrix1 <- matrix(NA,1,ncol(max_matrix))
  uq_num <- length(unique(max_matrix[1,]))
  if(substr(colnames(max_matrix)[1],1,1)=="H"){
    name_matrix1 <- colnames(max_matrix)
  }else{
    if(uq_num <=4){
      name_matrix1 <- colnames(max_matrix)
    }else{
      uq_len <- length(which(max_matrix[1,]==unique(max_matrix[1,])[1]| max_matrix[1,]==unique(max_matrix[1,])[2] | max_matrix[1,]==unique(max_matrix[1,])[3]))
      name_matrix1[1,1:uq_len] <- colnames(max_matrix)[which(max_matrix[1,]==unique(max_matrix[1,])[1] | max_matrix[1,]==unique(max_matrix[1,])[2] | max_matrix[1,]==unique(max_matrix[1,])[3])] 
    }
  }
  
  name_matrix11 <- matrix(NA,1,length(name_matrix1))
  for(nm1 in 1:length(name_matrix1)){
    name_matrix11[1,nm1] <- name_matrix1[nm1]
  }
  name_matrix1 <- name_matrix11
  ##Extraction of alleles that meet the filtration criteria
  num.col <- c()
  nc4 <- 1
  for(nc4 in 1:ncol(new.matrix1)){
    ##keep H
    if(substr(colnames(new.matrix1)[nc4],1,1)=="H"){
      next
    }else{
      nr3 <- 1
      for(nr3 in 1:(nrow(new.matrix1)-filter)){
        t <- 0
        linenum <- 1
        new.matrix2 <- matrix(0,filter+1,1)
        for(t in nr3:(nr3+filter)){
          if(new.matrix1[t,nc4]< mean(new.matrix1[t,])){
            if(new.matrix1[t,nc4]==0){
              new.matrix2[linenum,1] <- 1
            }
          }
          linenum <- linenum+1
        }
        
        if(sum(new.matrix2[,1])==filter+1){
          num.col <- c(num.col,nc4)
          break
        }
      }
    }
  }
  if(length(num.col)==0){
    new.matrix3 <- new.matrix1
  }else if(ncol(new.matrix1)-length(num.col)==1){
    new.matrix3 <- as.matrix(new.matrix1[,-num.col])
    colnames(new.matrix3) <- colnames(new.matrix1)[-num.col]
  }else{
    new.matrix3 <- new.matrix1[,-num.col]
  }
  if(ncol(new.matrix3) > 0){ 
    name_matrix2 <- matrix(NA,1,ncol(new.matrix3))
    name_matrix2 <- colnames(new.matrix3)
  }else{
    name_matrix2 <- c()
  }
  ##Merge alleles which are extracted through two standards
  if(length(name_matrix2)>0){
    name_matrix2 <- t(as.matrix(name_matrix2))
    name_matrix <- union(name_matrix1[1,which(!is.na(name_matrix1[1,]))],name_matrix2[1,which(!is.na(name_matrix2[1,]))])
  }else{
    name_matrix <- name_matrix1[1,which(!is.na(name_matrix1[1,]))]
  }
  name_matrix <- t(as.matrix(name_matrix))
  
  ##The names are merged if the top three of name is identical 
  uq_len1 <- ncol(name_matrix)
  new_name_matrix <- matrix(NA,uq_len1,1)
  tmp <- 1
  for(tmp in 1:uq_len1){
    len <- length(strsplit(name_matrix[tmp],':',fixed = TRUE)[[1]])
    if(len > 3){
      new_name_matrix[tmp,1] <- paste(strsplit(name_matrix[tmp],':',fixed = TRUE)[[1]][1:3],collapse = ":")
    }else{
      new_name_matrix[tmp,1] <- name_matrix[tmp]
    }
  }
  
  T <- 1
  for(T in 1:length(unique(new_name_matrix[,1]))){
    Tlines <-  which(new_name_matrix[,1]==unique(new_name_matrix[,1])[T])
    out_alleles_II[T,which(colnames(out_alleles_II)==alleles_group_II[g])] <- name_matrix[1,Tlines[1]]
  }
}

out_alleles <- cbind(out_alleles_I,out_alleles_II)
tmp_matrix <- cbind(tmp_matrix_I,tmp_matrix_II)

new_out_alleles <- matrix(NA,length(which(!is.na(out_alleles[,1])))+1,1)
if(nrow(new_out_alleles)>1){
  new_out_alleles[1,1] <- colnames(out_alleles)[1]
  new_out_alleles[2:nrow(new_out_alleles),1] <- out_alleles[which(!is.na(out_alleles[,1])),1]
}else{
  new_out_alleles[1,1] <- colnames(out_alleles)[1]
}

i <- 2
for(i in 2:ncol(out_alleles)){
  new_out_alleles1 <- matrix(NA,length(which(!is.na(out_alleles[,i])))+1,1)
  new_out_alleles1[1,1] <- colnames(out_alleles)[i]
  if(nrow(new_out_alleles1)>1){
    new_out_alleles1[2:nrow(new_out_alleles1),1] <- out_alleles[which(!is.na(out_alleles[,i])),i]
  }
  new_out_alleles <- matrix(rbind(new_out_alleles,new_out_alleles1))
}

write.table(new_out_alleles,file=out_filename,row.names = FALSE,na="",col.names = FALSE,quote = F)
